import { hasOwnProperty } from '../util/generalUtil.js';
import { AbstractDataSource } from './AbstractDataSource.js';
/**
 * Uses the {@link https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API Fetch API} to request the data from an api endpoint via HTTP POST requests
 */
export class FetchApiDataSource extends AbstractDataSource {
    url;
    options;
    /**
     * Create a new data source to fetch your paginated table data from an api endpoint.
     *
     * The endpoint will receive HTTP POST requests with a json body consisting of an {@link PaginatedListRequest}.
     * Your server has to answer with a {@link PaginatedListResponse} json body.
     * @param url a url to your api endpoint, for example `/api/users/list`
     * @param options [request init options](https://developer.mozilla.org/en-US/docs/Web/API/fetch#options) which get passed directly to fetch
     */
    constructor(url, options = {}) {
        super();
        this.url = url;
        this.options = options;
    }
    requestData(data) {
        fetch(this.url, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify(data),
            ...this.options
        }).then(async (response) => {
            let resData = undefined;
            try {
                resData = await response.json();
                // eslint-disable-next-line no-empty
            }
            catch (ignored) { }
            if (response.ok && typeof resData !== 'undefined') {
                this.updateStatus({
                    data: resData
                });
            }
            else {
                const errorMessage = typeof resData !== 'undefined' && hasOwnProperty(resData, 'message')
                    ? String(resData.message)
                    : 'Unknown network error';
                const error = new Error(errorMessage);
                if (resData) {
                    Object.assign(error, resData);
                }
                this.updateStatus({
                    error: error
                });
            }
        });
    }
}
