import { useQuery } from '@sveltestack/svelte-query';
import { derived } from 'svelte/store';
/**
 * Uses the Svelte Query library to fetch your paginated table data.
 */
export class SvelteQueryDataSource {
    apiFunction;
    additionalQueryOptions;
    dataQuery;
    dataObserver;
    queryKey;
    /**
     * Create a new data source to fetch your paginated table data from an api endpoint
     * using Svelte Query in the background to manage requests, caching and more.
     *
     * @param apiFunction the api function which actually fetches your data, passed through to Svelte Query as the [query function](https://sveltequery.vercel.app/guides/query-functions).
     * A query function can be literally any function that returns a promise. The promise that is returned should either resolve the data or throw an error.
     * @param additionalQueryOptions additional options passed to Svelte Query
     *
     * @see {@link https://sveltequery.vercel.app/reference/useQuery}
     */
    constructor(apiFunction, additionalQueryOptions = {}) {
        this.apiFunction = apiFunction;
        this.additionalQueryOptions = additionalQueryOptions;
        this.dataQuery = this.useNoopQuery();
        this.dataObserver = derived(this.dataQuery, ($dataQuery) => $dataQuery);
    }
    getQueryObserver() {
        return this.dataObserver;
    }
    init(config) {
        this.queryKey = `dataTable-${config.type}`;
    }
    requestData(data) {
        if (typeof this.queryKey === 'undefined') {
            throw new Error('Svelte-Query data source was not properly initialized before requesting data');
        }
        this.dataQuery.setOptions([this.queryKey, data], this.wrapApiFunction(), {
            enabled: true,
            keepPreviousData: true,
            ...this.additionalQueryOptions
        });
    }
    setApiFunction(apiFunction) {
        this.apiFunction = apiFunction;
    }
    setEnabled(enabled) {
        this.dataQuery.setEnabled(enabled);
    }
    updateQueryOptions(queryOptions) {
        this.additionalQueryOptions = {
            ...this.additionalQueryOptions,
            ...queryOptions
        };
        this.dataQuery.updateOptions(this.additionalQueryOptions);
    }
    useNoopQuery() {
        return useQuery([], () => undefined, {
            enabled: false
        });
    }
    wrapApiFunction() {
        return ({ queryKey }) => {
            return this.apiFunction(queryKey[1]);
        };
    }
}
